import time
import urllib2
import unittest
from xml.dom.minidom import parseString

from OpenTokSDK import OpenTokSDK, OpenTokException


class TestPythonSDK(unittest.TestCase):

    api_key = -1
    api_secret = ""
    o = None

    def setUp(self):
        self.o = OpenTokSDK(self.api_key, self.api_secret)

    def test_token_incorrect_key(self):
        different_session_id = "1_MX4yMDc1MjM4MX5-V2VkIEp1bCAxNyAxMDozMjoyMCBQRFQgMjAxM34wLjUyNTU2Mzd-" 
        try:
            t = self.o.generate_token(different_session_id)
            raise AssertionError('Python SDK tests: Token with incorrect API key should be rejected')
        except OpenTokException as e:
            pass #expected

       

    def test_create_session(self):
        s = self.o.create_session()
        t = self.o.generate_token(s.session_id)
        self.assertIsNotNone(s.session_id, "Python SDK tests: create session (no params): did not return a session id")
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual(s.session_id, xml.getElementsByTagName('session_id')[0].childNodes[0].data, \
            "Python SDK tests: Session id not found")

        s = self.o.create_session('216.38.134.114')
        t = self.o.generate_token(s.session_id)
        self.assertIsNotNone(s.session_id, "Python SDK tests: create session (no params): did not return a session id")
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual(s.session_id, xml.getElementsByTagName('session_id')[0].childNodes[0].data, \
            "Python SDK tests: Session id not found")

    def test_num_output_streams(self):
        s = self.o.create_session('127.0.0.1', properties={"multiplexer.numOutputStreams": 0})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('0', xml.getElementsByTagName('numOutputStreams')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.numOutputStreams not set to 0')

        s = self.o.create_session('127.0.0.1', properties = {"multiplexer.numOutputStreams": 1})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('1', xml.getElementsByTagName('numOutputStreams')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.numOutputStreams not set to 1')

        s = self.o.create_session('127.0.0.1', properties = {"multiplexer.numOutputStreams": 5})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('5', xml.getElementsByTagName('numOutputStreams')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.numOutputStreams not set to 5')

        s = self.o.create_session('127.0.0.1', properties = {"multiplexer.numOutputStreams": 100})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('100', xml.getElementsByTagName('numOutputStreams')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.numOutputStreams not set to 100')

        s = self.o.create_session('127.0.0.1')
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual([], xml.getElementsByTagName('numOutputStreams'),
            'Python SDK tests: multiplexer.numOutputStreams should not be set')

    def test_switch_type(self):
        s = self.o.create_session(properties = {"multiplexer.switchType":  0})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('0', xml.getElementsByTagName('switchType')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.switchType not 0')

        s = self.o.create_session(properties = {"multiplexer.switchType":  1})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('1', xml.getElementsByTagName('switchType')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.switchType not 1')

        s = self.o.create_session()
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual([], xml.getElementsByTagName('switchType'),
            'Python SDK tests: multiplexer.switchType should not be set')

    def test_switch_timeout(self):
        s = self.o.create_session(properties = {"multiplexer.switchTimeout": 1200})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('1200', xml.getElementsByTagName('switchTimeout')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.switchTimeout not properly set (should be 1200)')

        s = self.o.create_session(properties = {"multiplexer.switchTimeout": 2000})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('2000', xml.getElementsByTagName('switchTimeout')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.switchTimeout not properly set (should be 2000)')

        s = self.o.create_session(properties = {"multiplexer.switchTimeout": 100000})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('100000', xml.getElementsByTagName('switchTimeout')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.switchType not properly set (should be 100000)')

    def test_p2p_preference(self):
        s = self.o.create_session(properties = {"p2p.preference": 'enabled'})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('enabled', xml.getElementsByTagName('preference')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.p2p_preference not enabled')

        s = self.o.create_session(properties = {"p2p.preference": 'disabled'})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('disabled', xml.getElementsByTagName('preference')[0].childNodes[0].data, \
            'Python SDK tests: multiplexer.p2p_preference not disabled')

    def test_echo_suppression(self):
        s = self.o.create_session(properties = {"echoSuppression.enabled": 'true'})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('True', xml.getElementsByTagName('enabled')[0].childNodes[0].data, \
            'Python SDK tests: echo suppression not showing enabled')

        s = self.o.create_session(properties = {"echoSuppression.enabled": 'false'})
        t = self.o.generate_token(s.session_id)
        xml = self.get_session_info(s.session_id, t)
        self.assertEqual('False', xml.getElementsByTagName('enabled')[0].childNodes[0].data, \
            'Python SDK tests: echo suppression not showing disabled')

    def test_roles(self):
        s = self.o.create_session()

        # default: publisher
        t = self.o.generate_token(s.session_id)
        xml = self.get_token_info(t)
        role = xml.getElementsByTagName('role')[0].childNodes[0].data
        role = role.strip()
        self.assertEqual('publisher', role, 'Python SDK tests: default role not publisher')
        self.assertNotEqual([], xml.getElementsByTagName('subscribe'), \
            'Python SDK tests: token default permissions should include subscribe')
        self.assertNotEqual([], xml.getElementsByTagName('publish'), \
            'Python SDK tests: token default permissions should include publish')
        self.assertNotEqual([], xml.getElementsByTagName('signal'), \
            'Python SDK tests: token default permissions should include signal')
        self.assertEqual([], xml.getElementsByTagName('forceunpublish'), \
            'Python SDK tests: token default permissions should include forceunpublish')
        self.assertEqual([], xml.getElementsByTagName('forcedisconnect'), \
            'Python SDK tests: token default permissions should include forcedisconnect')
        self.assertEqual([], xml.getElementsByTagName('record'), \
            'Python SDK tests: token default permissions should include record')
        self.assertEqual([], xml.getElementsByTagName('playback'), \
            'Python SDK tests: token default permissions should include playback')

        # publisher
        t = self.o.generate_token(s.session_id, "publisher")
        xml = self.get_token_info(t)
        role = xml.getElementsByTagName('role')[0].childNodes[0].data
        role = role.strip()
        self.assertEqual('publisher', role, 'Python SDK tests: role not publisher')
        self.assertNotEqual([], xml.getElementsByTagName('subscribe'), \
            'Python SDK tests: token publisher permissions should include subscribe')
        self.assertNotEqual([], xml.getElementsByTagName('publish'), \
            'Python SDK tests: token publisher permissions should include publish')
        self.assertNotEqual([], xml.getElementsByTagName('signal'), \
            'Python SDK tests: token publisher permissions should include signal')
        self.assertEqual([], xml.getElementsByTagName('forceunpublish'), \
            'Python SDK tests: token publisher permissions should not include forceunpublish')
        self.assertEqual([], xml.getElementsByTagName('forcedisconnect'), \
            'Python SDK tests: token publisher permissions should not include forcedisconnect')
        self.assertEqual([], xml.getElementsByTagName('record'), \
            'Python SDK tests: token publisher permissions should not include record')
        self.assertEqual([], xml.getElementsByTagName('playback'), \
            'Python SDK tests: token publisher permissions should not include playback')

        # moderator
        t = self.o.generate_token(s.session_id, "moderator")
        xml = self.get_token_info(t)
        role = xml.getElementsByTagName('role')[0].childNodes[0].data
        role = role.strip()
        self.assertEqual('moderator', role, 'Python SDK tests: role not moderator')
        self.assertNotEqual([], xml.getElementsByTagName('subscribe'), \
            'Python SDK tests: token moderator permissions should include subscribe')
        self.assertNotEqual([], xml.getElementsByTagName('publish'), \
            'Python SDK tests: token moderator permissions should include publish')
        self.assertNotEqual([], xml.getElementsByTagName('signal'), \
            'Python SDK tests: token moderator permissions should include signal')
        self.assertNotEqual([], xml.getElementsByTagName('forceunpublish'), \
            'Python SDK tests: token moderator permissions should include forceunpublish')
        self.assertNotEqual([], xml.getElementsByTagName('forcedisconnect'), \
            'Python SDK tests: token moderator permissions should include forcedisconnect')
        self.assertNotEqual([], xml.getElementsByTagName('record'), \
            'Python SDK tests: token moderator permissions should include record')
        self.assertNotEqual([], xml.getElementsByTagName('playback'), \
            'Python SDK tests: token moderator permissions should include playback')

        # subscriber
        t = self.o.generate_token(s.session_id, "subscriber")
        xml = self.get_token_info(t)
        role = xml.getElementsByTagName('role')[0].childNodes[0].data
        role = role.strip()
        self.assertEqual('subscriber', role, 'Python SDK tests: role not subscriber')
        self.assertNotEqual([], xml.getElementsByTagName('subscribe'), \
            'Python SDK tests: token subscriber permissions should include subscribe')
        self.assertEqual([], xml.getElementsByTagName('publish'), \
            'Python SDK tests: token subscriber permissions should not include publish')
        self.assertEqual([], xml.getElementsByTagName('signal'), \
            'Python SDK tests: token subscriber permissions should not include signal')
        self.assertEqual([], xml.getElementsByTagName('forceunpublish'), \
            'Python SDK tests: token subscriber permissions should not include forceunpublish')
        self.assertEqual([], xml.getElementsByTagName('forcedisconnect'), \
            'Python SDK tests: token subscriber permissions should not include forcedisconnect')
        self.assertEqual([], xml.getElementsByTagName('record'), \
            'Python SDK tests: token subscriber permissions should not include record')
        self.assertEqual([], xml.getElementsByTagName('playback'), \
            'Python SDK tests: token subscriber permissions should not include playback')

        # garbage data
        try:
            t = self.o.generate_token(s.session_id, 'ads')
            raise AssertionError('Python SDK tests: invalid role should be rejected')

        except OpenTokException:
            pass # expected

    def test_old_session(self):
        s_id = '1abc70a34d069d2e6a1e565f3958b5250b435e32'
        t = self.o.generate_token(s_id)
        xml = self.get_token_info(t)
        self.assertEqual(s_id, xml.getElementsByTagName('session_id')[0].childNodes[0].data, \
            'Python SDK test: creating token with old session id failed')

    def test_expire_time(self):
        s = self.o.create_session()

        # past
        tm = time.time() - 1000
        try:
            t = self.o.generate_token(s.session_id, expire_time=tm)
            raise AssertionError('Python SDK tests: expire time in past should be rejected')
        except OpenTokException:
            pass

        # near future
        tm = time.time() + 6 * 24 * 60 * 60
        t = self.o.generate_token(s.session_id, expire_time=tm)
        xml = self.get_token_info(t)
        self.assertEqual(str(int(tm)), xml.getElementsByTagName('expire_time')[0].childNodes[0].data,\
            'Python SDK tests: expire time not set')

        # far future
        tm = time.time() + 8 * 24 * 60 * 60
        try:
            t = self.o.generate_token(s.session_id, expire_time=tm)
            raise AssertionError('Python SDK tests: expire time more than 7 days away should be rejected')
        except OpenTokException:
            pass

        # garbage data
        try:
            t = self.o.generate_token(s.session_id, expire_time="asdf")
            raise AssertionError('Python SDK tests: non numberic expire time should be rejected')
        except OpenTokException:
            pass

    def test_connection_data(self):
        s = self.o.create_session()

        test_string = 'test data'
        t = self.o.generate_token(s.session_id, connection_data=test_string)
        xml = self.get_token_info(t)
        self.assertEqual(test_string, xml.getElementsByTagName('connection_data')[0].childNodes[0].data,\
            'Python SDK tests: connection data not correct')

        # Should reject over 1000 characters of connection data
        test_string = 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa' + \
            'bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb' + \
            'cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc' + \
            'dddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd' + \
            'eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee' + \
            'eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee' + \
            'ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff' + \
            'gggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggggg' + \
            'hhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhh' + \
            'iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii' + \
            'jjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjj' + \
            'kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk' + \
            'llllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllll' + \
            'mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm' + \
            'nnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnn' + \
            'oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo'
        try:
            t = self.o.generate_token(s.session_id, connection_data=test_string)
            raise AssertionError('Python SDK tests: long connection data should be rejected')
        except OpenTokException:
            pass # expected

    def get_session_info(self, session_id, token):
        try:
            url = self.o.API_URL + '/session/' + session_id + '?extended=true'
            req = urllib2.Request(url, '', { 'X-TB-TOKEN-AUTH': token })
            response = urllib2.urlopen(req)
            dom = parseString(response.read())
            response.close()

            return dom
        except urllib2.HTTPError:
            print "HTTP Error: failed to send request"

    def get_token_info(self, token):
        try:
            url = self.o.API_URL + '/token/validate'
            req = urllib2.Request(url, '', { 'X-TB-TOKEN-AUTH': token })
            response = urllib2.urlopen(req)
            dom = parseString(response.read())
            response.close()

            return dom
        except urllib2.HTTPError:
            print "HTTP Error: failed to send request"

    def assertIsNotNone(self, obj, msg):
        if obj is None:
            raise AssertionError(msg)

    def assertIsNone(self, obj, msg):
        if obj is not None:
            raise AssertionError(msg)

if __name__ == '__main__':
        unittest.main()



"""
OpenTok Python Library v0.91.0
http://www.tokbox.com/

Copyright 2011, TokBox, Inc.
"""


import urllib
import urllib2
import datetime
import calendar
import time
import hmac
import hashlib
import base64
import random


TIMEOUT = 10


class OpenTokException(BaseException):
    """Generic OpenTok Error. All other errors extend this."""
    pass


class RequestError(OpenTokException):
    """Indicates an error during the request. Most likely an error connecting to
    the OpenTok API servers. (HTTP 500 error).
    """
    pass


class AuthError(OpenTokException):
    """Indicates that the problem was likely with credentials. Check your API
    key and API secret and try again.
    """
    pass


class SessionProperties(object):
    echoSuppression_enabled = None
    multiplexer_numOutputStreams = None
    multiplexer_switchType = None
    multiplexer_switchTimeout = None
    p2p_preference = "p2p.preference" 

    def __iter__(self):
        d = {
            'echoSuppression.enabled': self.echoSuppression_enabled,
            'multiplexer.numOutputStreams': self.multiplexer_numOutputStreams,
            'multiplexer.switchType': self.multiplexer_switchType,
            'multiplexer.switchTimeout': self.multiplexer_switchTimeout,
            'p2p.preference': self.p2p_preference,
        }
        return d.iteritems()


class RoleConstants:
    """List of valid roles for a token."""
    SUBSCRIBER = 'subscriber' #Can only subscribe
    PUBLISHER = 'publisher'   #Can publish, subscribe, and signal
    MODERATOR = 'moderator'   #Can do the above along with  forceDisconnect and forceUnpublish


class OpenTokSession(object):

    def __init__(self, session_id):
        self.session_id = session_id

class OpenTokSDK(object):
    """Use this SDK to create tokens and interface with the server-side portion
    of the Opentok API.
    """
    TOKEN_SENTINEL = 'T1=='
    API_URL = 'http://api.opentok.com'

    def __init__(self, api_key, api_secret):
        self.api_key = api_key
        self.api_secret = api_secret.strip()

    def generate_token(self, session_id, role=None, expire_time=None, connection_data=None, **kwargs):
        """
        Generate a token which is passed to the JS API to enable widgets to connect to the Opentok api.
        session_id: Specify a session_id to make this token only valid for that session_id.
        role: One of the constants defined in RoleConstants. Default is publisher, look in the documentation to learn more about roles.
        expire_time: Integer timestamp. You can override the default token expire time of 24h by choosing an explicit expire time. Can be up to 7d after create_time.
        """
        create_time = datetime.datetime.utcnow()
        if not session_id:
            raise OpenTokException('Null or empty session ID are not valid')
        sub_session_id = session_id[2:]
        decoded_session_id=""
        for i in range(0,3):
            new_session_id = sub_session_id+("="*i)
            new_session_id = new_session_id.replace("-","+").replace("_","/")
            try:
                decoded_session_id = base64.b64decode(new_session_id)
                if "~" in decoded_session_id:
                    break
            except TypeError:
                pass
        try:
            if decoded_session_id.split('~')[1]!=str(self.api_key):
                raise OpenTokException("An invalid session ID was passed")
        except Exception as e:
            raise OpenTokException("An invalid session ID was passed")

        if not role:
            role = RoleConstants.PUBLISHER

        if role != RoleConstants.SUBSCRIBER and \
                role != RoleConstants.PUBLISHER and \
                role != RoleConstants.MODERATOR:
            raise OpenTokException('%s is not a valid role' % role)

        data_params = dict(
            session_id=session_id,
            create_time=calendar.timegm(create_time.timetuple()),
            role=role,
        )
        if expire_time is not None:
            if isinstance(expire_time, datetime.datetime):
                data_params['expire_time'] = calendar.timegm(expire_time.timetuple())
            else:
                try:
                    data_params['expire_time'] = int(expire_time)
                except (ValueError, TypeError):
                    raise OpenTokException('Expire time must be a number')

            if data_params['expire_time'] < time.time():
                raise OpenTokException('Expire time must be in the future')

            if data_params['expire_time'] > time.time() + 2592000:
                raise OpenTokException('Expire time must be in the next 30 days')

        if connection_data is not None:
            if len(connection_data) > 1000:
                raise OpenTokException('Connection data must be less than 1000 characters')
            data_params['connection_data'] = connection_data

        data_params['nonce'] = random.randint(0,999999)
        data_string = urllib.urlencode(data_params, True)

        sig = self._sign_string(data_string, self.api_secret)
        token_string = '%s%s' % (self.TOKEN_SENTINEL, base64.b64encode('partner_id=%s&sig=%s:%s' % (self.api_key, sig, data_string)))
        return token_string

    def create_session(self, location='', properties={}, **kwargs):
        """Create a new session in the OpenTok API. Returns an OpenTokSession
        object with a session_id property. location: IP address of the user
        requesting the session. This is used for geolocation to choose which
        datacenter the session will live on. properties: An instance of the
        SessionProperties object. Fill in the fields that you are interested in
        to use features of the groups API. Look in the documentation for more
        details. Also accepts any dict-like object.
        """
        #ip_passthru is a deprecated argument and has been replaced with location
        if 'ip_passthru' in kwargs:
            location = kwargs['ip_passthru']
        params = dict(api_key=self.api_key)
        params['location'] = location
        params.update(properties)
        dom = ''
        try:
            dom = self._do_request('/session/create', params)
        except RequestError:
            raise
        except Exception, e:
            raise RequestError('Failed to create session: %s' % str(e)  )

        try:
            error = dom.getElementsByTagName('error')
            if error:
                error = error[0]
                raise AuthError('Failed to create session (code=%s): %s' % (error.attributes['code'].value, error.firstChild.attributes['message'].value))

            session_id = dom.getElementsByTagName('session_id')[0].childNodes[0].nodeValue
            return OpenTokSession(session_id)
        except Exception, e:
            raise OpenTokException('Failed to generate session: %s' % str(e))


    def _sign_string(self, string, secret):
        return hmac.new(secret, string.encode('utf-8'), hashlib.sha1).hexdigest()

    def _do_request(self, url, params):
        import xml.dom.minidom as xmldom

        if '_token' in params: #Do token auth if _token is present, partner auth normally
            auth_header = ('X-TB-TOKEN-AUTH', params['_token'])
            del params['_token']
        else:
            auth_header = ('X-TB-PARTNER-AUTH', '%s:%s' % (self.api_key, self.api_secret))

        method = 'POST' if params else 'GET'
        data_string = urllib.urlencode(params, True)

        context_source = [
            ('method', method),
            ('Content-Type', 'application/x-www-form-urlencoded'),
            ('Content-Length', len(data_string)),
            auth_header
        ]

        req_string = self.API_URL + url
        try:
            opener = urllib2.build_opener()
            opener.addheaders = context_source
            if data_string:
                request = urllib2.Request(url=req_string, data=data_string)
            else: #GET if no data_string
                request = urllib2.Request(url=req_string)
            try:
                response = opener.open(request, timeout=TIMEOUT)
            except TypeError: #Python2.6 added the timeout keyword, if it doesn't get accepted, try without it
                response = opener.open(request)

            dom = xmldom.parseString(response.read())
            response.close()
        except urllib2.HTTPError, e:
            raise RequestError('Failed to send request: %s' % str(e))

        return dom



from os.path import abspath, dirname, join, normpath

from setuptools import setup


setup(

    # Basic package information:
    name = 'opentok',
    version = '0.91.2',
    py_modules = ['OpenTokSDK'],

    # Packaging options:
    zip_safe = False,
    include_package_data = True,

    # Metadata for PyPI:
    author = 'TokBox, Inc.',
    author_email = 'support@tokbox.com',
    #license = '?',
    url = 'https://github.com/opentok/Opentok-Python-SDK',
    keywords = 'video chat tokbox tok opentok python media',
    description = 'A python wrapper for the OpenTok video chat APIs.',
    long_description = open(normpath(join(dirname(abspath(__file__)),
        'README'))).read(),
    classifiers = [
        'Intended Audience :: Developers',
        'Topic :: Communications',
        'Topic :: Communications :: Chat',
        'Topic :: Communications :: Conferencing',
        'Topic :: Multimedia :: Video :: Capture',
        'Topic :: Multimedia :: Video :: Display',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

)
